@@ -8,58 +8,6 @@ SUCCESS = '\u001b[32m'
 WARN = '\u001b[31m'
 RESET = '\u001b[0m'
 
-MEDIA = {
-    '.mp4': {
-        'type': 'video/mp4',
-        'category': {
-            'tweet': 'tweet_video',
-            'dm': 'dm_video',
-        }
-    },
-    '.mov': {
-        'type': 'video/quicktime',
-        'category': {
-            'tweet': 'tweet_video',
-            'dm': 'dm_video',
-        }
-    },
-    '.png': {
-        'type': 'image/png',
-        'category': {
-            'tweet': 'tweet_image',
-            'dm': 'dm_image',
-        }
-    },
-    '.jpg': {
-        'type': 'image/jpeg',
-        'category': {
-            'tweet': 'tweet_image',
-            'dm': 'dm_image',
-        }
-    },
-    '.jpeg': {
-        'type': 'image/jpeg',
-        'category': {
-            'tweet': 'tweet_image',
-            'dm': 'dm_image',
-        }
-    },
-    '.jfif': {
-        'type': 'image/jpeg',
-        'category': {
-            'tweet': 'tweet_image',
-            'dm': 'dm_image',
-        }
-    },
-    '.gif': {
-        'type': 'image/gif',
-        'category': {
-            'tweet': 'tweet_gif',
-            'dm': 'dm_gif',
-        }
-    },
-}
-
 content_settings = {
     'include_mention_filter': True,
     'include_nsfw_user_flag': True,
@@ -67,7 +15,7 @@ content_settings = {
     'include_ranked_timeline': True,
     'include_alt_text_compose': True,
     'display_sensitive_media': True,
-    'protected': True,
+    'protected': False,
     'discoverable_by_email': False,
     'discoverable_by_mobile_phone': False,
     'allow_dms_from': 'following',  ## {'all'}
@@ -113,6 +61,6 @@ follow_settings = {
 }
 
 account_search_settings = {
-    "optInFiltering": False,
-    "optInBlocking": True,
+    "optInFiltering": True,  # filter out nsfw content
+    "optInBlocking": True,  # filter out blocked accounts
 }
